"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ern_core_1 = require("ern-core");
const ern_container_gen_1 = require("ern-container-gen");
const fs_extra_1 = __importDefault(require("fs-extra"));
const path_1 = __importDefault(require("path"));
const xcode_ern_1 = __importDefault(require("xcode-ern"));
const lodash_1 = __importDefault(require("lodash"));
const fs_readdir_recursive_1 = __importDefault(require("fs-readdir-recursive"));
const semver_1 = __importDefault(require("semver"));
const ROOT_DIR = process.cwd();
const PATH_TO_TEMPLATES_DIR = path_1.default.join(__dirname, 'templates');
const PATH_TO_HULL_DIR = path_1.default.join(__dirname, 'hull');
class IosGenerator {
    get name() {
        return 'IosGenerator';
    }
    get platform() {
        return 'ios';
    }
    async generate(config) {
        return (0, ern_container_gen_1.generateContainer)(config, {
            fillContainerHull: this.fillContainerHull.bind(this),
            postCopyRnpmAssets: this.addResources.bind(this),
        });
    }
    async addResources(config) {
        const containerProjectPath = path_1.default.join(config.outDir, 'ElectrodeContainer.xcodeproj', 'project.pbxproj');
        const containerIosProject = await this.getIosContainerProject(containerProjectPath);
        const containerResourcesPath = path_1.default.join(config.outDir, 'ElectrodeContainer', 'Resources');
        // Get all resources files from the 'Resources' directory
        // Some processing is done here to properly handle `.xcassets`
        // files which are directories containing assets.
        // But we don't want to add independent files from this directory
        // to the pbxproj, but only the top level `.xcassets` directory.
        //
        // For example, an  `xcassets` directory might look like this
        //
        // Media.xcassets
        // ├── Contents.json
        // ├── Face-ID.imageset
        // │   ├── Contents.json
        // │   └── faceid.pdf
        // ├── IconUser.imageset
        // │   ├── Contents.json
        // │   └── IconUser.pdf
        //
        // Which will result in recursive readir call returning the
        // following paths
        //
        // Media.xcassets/Contents.json
        // Media.xcassets/Face-ID.imageset/Contents.json
        // Media.xcassets/Face-ID.imageset/faceid.pdf
        // ...
        //
        // Each of these individual files would then wrongly be
        // added to the pbxproj as resources.
        // However we just want top level directory Media.xcassets
        // to be added to the pbxproj, thus the special processing.
        const resourceFiles = lodash_1.default.uniq((0, fs_readdir_recursive_1.default)(containerResourcesPath).map((f) => f.replace(/(\.xcassets)(\/.+)$/, '$1')));
        resourceFiles.forEach((resourceFile) => {
            containerIosProject.addResourceFile(path_1.default.join('Resources', resourceFile), null, containerIosProject.findPBXGroupKey({ name: 'Resources' }));
        });
        fs_extra_1.default.writeFileSync(containerProjectPath, containerIosProject.writeSync());
    }
    async fillContainerHull(config) {
        var _a, _b, _c, _d, _e, _f, _g;
        if (process.platform !== 'darwin' && !((_a = config === null || config === void 0 ? void 0 : config.iosConfig) === null || _a === void 0 ? void 0 : _a.skipInstall)) {
            throw new Error(`Full iOS Container generation with pod installation is only supported on macOS.
You can set 'iosConfig.skipInstall' in container generator config to skip the installation step.
Or set --skipInstall if using the create-container command.`);
        }
        if ((_b = config === null || config === void 0 ? void 0 : config.iosConfig) === null || _b === void 0 ? void 0 : _b.skipInstall) {
            ern_core_1.log.info(`skipInstall option is set. 'yarn install' and 'pod install' won't be run after container generation.
Make sure to run these commands before building the container.`);
        }
        const pathSpec = {
            outputDir: config.outDir,
            projectHullDir: path_1.default.join(PATH_TO_HULL_DIR, '{.*,*}'),
            rootDir: ROOT_DIR,
        };
        const reactNativePlugin = lodash_1.default.find(config.plugins, (p) => p.name === 'react-native');
        if (!reactNativePlugin) {
            throw new Error('react-native was not found in plugins list !');
        }
        if (!reactNativePlugin.version) {
            throw new Error('react-native plugin does not have a version !');
        }
        const mustacheView = {};
        mustacheView.jsMainModuleName = config.jsMainModuleName || 'index';
        mustacheView.iosDeploymentTarget =
            (_d = (_c = config.iosConfig) === null || _c === void 0 ? void 0 : _c.deploymentTarget) !== null && _d !== void 0 ? _d : ern_core_1.iosUtil.getDefaultIosDeploymentTarget(reactNativePlugin.version);
        (0, ern_core_1.injectReactNativeVersionKeysInObject)(mustacheView, reactNativePlugin.version);
        const projectSpec = {
            deploymentTarget: mustacheView.iosDeploymentTarget,
            projectName: 'ElectrodeContainer',
        };
        await ern_core_1.kax
            .task('Preparing Native Dependencies Injection')
            .run(this.buildiOSPluginsViews(config.plugins, mustacheView));
        await ern_core_1.kax
            .task('Preparing API Implementations Injection')
            .run(this.buildApiImplPluginViews(config.plugins, config.composite, mustacheView, projectSpec));
        if (semver_1.default.gte(reactNativePlugin.version, '0.61.0')) {
            ern_core_1.shell.pushd(config.outDir);
            try {
                await ern_core_1.yarn.init();
                // Add @react-native-community/cli-platform-ios because
                // it contains the scripts needed for native modules pods linking
                // look in composite to match proper version
                const compositeNodeModulesPath = path_1.default.join(config.composite.path, 'node_modules');
                const cliPlatformIosPkg = '@react-native-community/cli-platform-ios';
                const cliPlatformIosPkgVersion = (await (0, ern_core_1.readPackageJson)(path_1.default.join(compositeNodeModulesPath, cliPlatformIosPkg))).version;
                await ern_core_1.yarn.add(ern_core_1.PackagePath.fromString(`${cliPlatformIosPkg}@${cliPlatformIosPkgVersion}`));
                await ern_core_1.yarn.add(ern_core_1.PackagePath.fromString(`react-native@${reactNativePlugin.version}`));
                //
                // Add all native dependencies to package.json dependencies so that
                // !use_native_modules can detect them to add their pod to the Podfile
                const dependencies = await config.composite.getNativeDependencies({});
                const resDependencies = [
                    ...dependencies.thirdPartyInManifest,
                    ...dependencies.thirdPartyNotInManifest,
                ];
                const addDependencies = {};
                resDependencies.forEach((p) => {
                    addDependencies[p.name] = p.version;
                });
                //
                // Create package.json in container directory root
                // so that native modules pods can be resolved
                // by use_native_modules! RN ruby script
                const pjsonObj = {
                    dependencies: addDependencies,
                    name: 'container',
                    private: true,
                };
                await (0, ern_core_1.writePackageJson)(config.outDir, pjsonObj);
                //
                // Copy all native dependencies from composite node_modules
                // to container node_modules so that pods can be found local
                // to the container directory
                // [only for full iOS container generation]
                if (!((_e = config === null || config === void 0 ? void 0 : config.iosConfig) === null || _e === void 0 ? void 0 : _e.skipInstall)) {
                    const containerNodeModulesPath = path_1.default.join(config.outDir, 'node_modules');
                    ern_core_1.shell.mkdir('-p', containerNodeModulesPath);
                    resDependencies.forEach((p) => {
                        const depPath = p.basePath.match(/node_modules\/(.+)/)[1];
                        const targetPath = path_1.default.join(containerNodeModulesPath, depPath);
                        ern_core_1.shell.mkdir('-p', targetPath);
                        ern_core_1.shell.cp('-rf', path_1.default.join(p.basePath, '{.*,*}'), targetPath);
                    });
                }
                else {
                    ern_core_1.shell.rm('-rf', 'node_modules');
                }
            }
            finally {
                ern_core_1.shell.popd();
            }
        }
        const { iosProject, projectPath } = await ern_core_1.iosUtil.fillProjectHull(pathSpec, projectSpec, config.plugins, mustacheView, config.composite, config.iosConfig);
        await ern_core_1.kax
            .task('Adding MiniAppNavigationController Classes')
            .run(this.addMiniAppNavigationControllerClasses(config, iosProject));
        await ern_core_1.kax
            .task('Adding Native Dependencies Hooks')
            .run(this.addiOSPluginHookClasses(iosProject, config.plugins, config.outDir));
        fs_extra_1.default.writeFileSync(projectPath, iosProject.writeSync());
        if (semver_1.default.gte(reactNativePlugin.version, '0.61.0')) {
            // For full iOS container generation, run 'pod install'
            // and also add all essential node_modules (needed for the build)
            // to the container
            if (!((_f = config === null || config === void 0 ? void 0 : config.iosConfig) === null || _f === void 0 ? void 0 : _f.skipInstall)) {
                const podVersionArg = ern_core_1.config.get('podVersion')
                    ? `_${ern_core_1.config.get('podVersion')}_`
                    : '';
                const podRepoUpdateArg = ((_g = config === null || config === void 0 ? void 0 : config.iosConfig) === null || _g === void 0 ? void 0 : _g.podRepoUpdate)
                    ? '--repo-update'
                    : '';
                const podCmdArgs = [
                    podVersionArg,
                    'install',
                    podRepoUpdateArg,
                    '--verbose',
                ].filter((x) => x !== '');
                ern_core_1.shell.pushd(config.outDir);
                try {
                    await ern_core_1.kax
                        .task(`Running pod ${podCmdArgs.join(' ')}`)
                        .run(ern_core_1.childProcess.spawnp('pod', podCmdArgs));
                }
                finally {
                    ern_core_1.shell.popd();
                }
                //
                // Clean node_modules by only keeping the directories that are
                // needed for proper container build.
                ern_core_1.shell.pushd(config.outDir);
                try {
                    //
                    // Look in the Pods pbxproj for any references to some files
                    // kepts in some node_module subdirectory (basically react-native
                    // as well as all native modules)
                    const f = fs_extra_1.default.readFileSync('Pods/Pods.xcodeproj/project.pbxproj', {
                        encoding: 'utf8',
                    });
                    //
                    // Build an array of these directories
                    const re = RegExp('"?../node_modules/([^"]+)"?;', 'g');
                    const matches = [];
                    let match = re.exec(f);
                    while (match !== null) {
                        matches.push(match[1]);
                        match = re.exec(f);
                    }
                    const res = matches
                        .map((r) => r.split('/'))
                        .filter((x) => x[0] !== 'react-native')
                        .map((x) => x.join('/'))
                        .concat('react-native');
                    //
                    // Copy all retained directories from 'node_modules'
                    // to a new directory 'node_modules_light'
                    const nodeModulesLightDir = 'node_modules_light';
                    const nodeModulesDir = 'node_modules';
                    ern_core_1.shell.mkdir('-p', nodeModulesLightDir);
                    for (const b of res) {
                        ern_core_1.shell.mkdir('-p', path_1.default.join(nodeModulesLightDir, b));
                        ern_core_1.shell.cp('-Rf', path_1.default.join(nodeModulesDir, b, '{.*,*}'), path_1.default.join(nodeModulesLightDir, b));
                    }
                    //
                    // Replace the huge 'node_modules' directory with the skimmed one
                    ern_core_1.shell.rm('-rf', nodeModulesDir);
                    ern_core_1.shell.mv(nodeModulesLightDir, nodeModulesDir);
                    //
                    // Finally get rid of all android directories to further reduce
                    // overall 'node_modules' directory size, as they are not needed
                    // for iOS container builds.
                    ern_core_1.shell.rm('-rf', path_1.default.join(nodeModulesDir, '**/android'));
                    const codegenVersion = ern_core_1.iosUtil.getReactNativeCodegenVersion(reactNativePlugin.version);
                    if (codegenVersion) {
                        // Starting with React Native 0.64.0, code generation for Turbo Modules
                        // is performed using react-native-codegen package.
                        // The package entry point is invoked during Xcode build of FBReactNativeSpec
                        // iOS project (run as a build phase script).
                        // It therefore needs to be present in the node modules of the Container.
                        //
                        // TODO : Dig deeper to figure out if we could instead call this script during
                        // container generation and get rid of the the build script phase after doing
                        // that, so that we don't clutter node_modules of the container pushed to git
                        // and that Node does not have to be a requirement to build containers.
                        await this.addReactNativeCodegen(config.outDir, codegenVersion);
                    }
                }
                finally {
                    ern_core_1.shell.popd();
                }
            }
        }
    }
    async addReactNativeCodegen(targetDir, version) {
        ern_core_1.log.debug('Adding react-native-codegen package');
        const tmpDir = (0, ern_core_1.createTmpDir)();
        ern_core_1.shell.pushd(tmpDir);
        try {
            await ern_core_1.yarn.init();
            await ern_core_1.yarn.add(ern_core_1.PackagePath.fromString(`react-native-codegen@${version}`));
            // mkdirp and invariant are also needed
            await ern_core_1.yarn.add(ern_core_1.PackagePath.fromString('mkdirp'));
            await ern_core_1.yarn.add(ern_core_1.PackagePath.fromString('invariant'));
            // yargs is needed since this package is missing from generate-specs-cli script. - https://github.com/facebook/react-native/issues/36315
            // Resolved from RN 0.72.0-rc.0 to prevent script failures - https://github.com/facebook/react-native/commit/5093e557efac35632d99cd2094527f9e01cbc519
            await ern_core_1.yarn.add(ern_core_1.PackagePath.fromString('yargs'));
            ern_core_1.shell.rm('-rf', [
                'package.json',
                'yarn.lock',
                'node_modules/.yarn-integrity',
            ]);
            ern_core_1.shell.cp('-Rf', path_1.default.join(tmpDir, 'node_modules'), targetDir);
        }
        finally {
            ern_core_1.shell.popd();
        }
    }
    // Code to keep backward compatibility
    switchToOldDirectoryStructure(pluginSourcePath, tail) {
        // This is to check if the api referenced during container generation is created using the old or new directory structure to help keep the backward compatibility.
        const pathToSwaggersAPIs = path_1.default.join('IOS', 'IOS', 'Classes', 'SwaggersAPIs');
        if (path_1.default.dirname(tail) === 'IOS' &&
            fs_extra_1.default.pathExistsSync(path_1.default.join(pluginSourcePath, path_1.default.dirname(pathToSwaggersAPIs)))) {
            return true;
        }
        return false;
    }
    async buildiOSPluginsViews(plugins, mustacheView) {
        mustacheView.plugins = await (0, ern_container_gen_1.generatePluginsMustacheViews)(plugins, 'ios');
    }
    async addiOSPluginHookClasses(containerIosProject, plugins, outDir) {
        for (const plugin of plugins) {
            if (plugin.name === 'react-native') {
                continue;
            }
            const pluginConfig = await ern_core_1.manifest.getPluginConfig(plugin, 'ios');
            if (!pluginConfig) {
                ern_core_1.log.warn(`${plugin.name} does not have any injection configuration for ios platform`);
                continue;
            }
            const { pluginHook } = pluginConfig;
            if (pluginHook === null || pluginHook === void 0 ? void 0 : pluginHook.name) {
                if (!pluginConfig.path) {
                    throw new Error('No plugin config path was set. Cannot proceed.');
                }
                const pluginConfigPath = pluginConfig.path;
                const pathToCopyPluginHooksTo = path_1.default.join(outDir, 'ElectrodeContainer');
                ern_core_1.log.debug(`Adding ${pluginHook.name}.h`);
                const pathToPluginHookHeader = path_1.default.join(pluginConfigPath, `${pluginHook.name}.h`);
                ern_core_1.shell.cp(pathToPluginHookHeader, pathToCopyPluginHooksTo);
                containerIosProject.addHeaderFile(`${pluginHook.name}.h`, { public: true }, containerIosProject.findPBXGroupKey({ name: 'ElectrodeContainer' }));
                ern_core_1.log.debug(`Adding ${pluginHook.name}.m`);
                const pathToPluginHookSource = path_1.default.join(pluginConfigPath, `${pluginHook.name}.m`);
                ern_core_1.shell.cp(pathToPluginHookSource, pathToCopyPluginHooksTo);
                containerIosProject.addSourceFile(`${pluginHook.name}.m`, null, containerIosProject.findPBXGroupKey({ name: 'ElectrodeContainer' }));
            }
        }
    }
    async getIosContainerProject(containerProjectPath) {
        const containerProject = xcode_ern_1.default.project(containerProjectPath);
        return new Promise((resolve, reject) => {
            containerProject.parse((err) => {
                if (err) {
                    reject(err);
                }
                resolve(containerProject);
            });
        });
    }
    async buildApiImplPluginViews(plugins, composite, mustacheView, projectSpec) {
        for (const plugin of plugins) {
            const pluginConfig = await ern_core_1.manifest.getPluginConfig(plugin, 'ios', projectSpec.projectName);
            if (!pluginConfig) {
                continue;
            }
            if (await ern_core_1.utils.isDependencyPathNativeApiImpl(plugin.basePath)) {
                (0, ern_container_gen_1.populateApiImplMustacheView)(plugin.basePath, mustacheView, true);
            }
        }
        if (mustacheView.apiImplementations) {
            mustacheView.hasApiImpl = true;
            for (const api of mustacheView.apiImplementations) {
                if (api.hasConfig) {
                    mustacheView.hasAtleastOneApiImplConfig = true;
                    break;
                }
            }
        }
    }
    async addMiniAppNavigationControllerClasses(config, containerIosProject) {
        const partialProxy = (name) => {
            return fs_extra_1.default.readFileSync(path_1.default.join(PATH_TO_TEMPLATES_DIR, `${name}.mustache`), 'utf8');
        };
        ern_core_1.log.debug('Creating miniapps config');
        const configFileName = `MiniAppsConfig.swift`;
        const pathToMiniAppsConfigMustacheTemplate = path_1.default.join(PATH_TO_TEMPLATES_DIR, 'MiniAppsConfig.mustache');
        const pathToOutputConfigFile = path_1.default.join(config.outDir, 'ElectrodeContainer', 'MiniAppNavigationControllers', configFileName);
        const mustacheView = {};
        mustacheView.miniApps = await config.composite.getMiniApps();
        await ern_core_1.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(pathToMiniAppsConfigMustacheTemplate, mustacheView, pathToOutputConfigFile, partialProxy);
        const relativePathToConfigFile = path_1.default.join('MiniAppNavigationControllers', configFileName);
        containerIosProject.addFile(relativePathToConfigFile, containerIosProject.findPBXGroupKey({
            name: 'MiniAppNavigationControllers',
        }));
        containerIosProject.addSourceFile(relativePathToConfigFile, null, containerIosProject.findPBXGroupKey({
            name: 'MiniAppNavigationControllers',
        }));
        ern_core_1.log.debug('Creating miniapp nav controllers');
        const compositeMiniApps = await config.composite.getMiniApps();
        for (const miniApp of compositeMiniApps) {
            const navControllerFileName = `${miniApp.pascalCaseName}NavigationController.swift`;
            ern_core_1.log.debug(`Creating ${navControllerFileName}`);
            const pathToMiniAppNavControllerMustacheTemplate = path_1.default.join(PATH_TO_TEMPLATES_DIR, 'MiniAppNavigationController.mustache');
            const pathToOutputNavControllerFile = path_1.default.join(config.outDir, 'ElectrodeContainer', 'MiniAppNavigationControllers', navControllerFileName);
            await ern_core_1.mustacheUtils.mustacheRenderToOutputFileUsingTemplateFile(pathToMiniAppNavControllerMustacheTemplate, miniApp, pathToOutputNavControllerFile, partialProxy);
            const relativePathToNavControllerFile = path_1.default.join('MiniAppNavigationControllers', navControllerFileName);
            containerIosProject.addFile(relativePathToNavControllerFile, containerIosProject.findPBXGroupKey({
                name: 'MiniAppNavigationControllers',
            }));
            containerIosProject.addSourceFile(relativePathToNavControllerFile, null, containerIosProject.findPBXGroupKey({
                name: 'MiniAppNavigationControllers',
            }));
        }
    }
}
exports.default = IosGenerator;
//# sourceMappingURL=IosGenerator.js.map